/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.
Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useRef, useState, useEffect, useContext } from 'react';
import { Grid } from '@material-ui/core';
import CollectionAuthor from '../Components/CollectionAuthor.js';
import VerticalRuler from '../Components/VerticalRuler.js';
import CollectionDescription from '../Components/CollectionDescription.js';
import ResourceList from '../Components/ResourceList.js';
import CollectionCommentSection from '../Components/CollectionCommentSection.js';
import LoadingSpinner from '../Components/LoadingSpinner';
import styled from 'styled-components';
import DowloadButton from '../Components/CollectionDowloadButton.js';
import Breadcrumbs from "@material-ui/core/Breadcrumbs";
import { Link } from 'react-router-dom';
import FollowCollectionButton from '../Components/FollowCollectionButton';
import { Store } from '../Store.js';
import Button from '@material-ui/core/Button';
import { getRequest } from '../Components/HelperFunctions/getAxiosConfig.js';
import noAvatar from '../img/default_profile.png';
import { apiDomain } from '../env.js';

export default function CollectionPage(props) {
	const { state } = useContext(Store);

	const [error, setError] = useState(false)
	const [loading, setLoading] = useState(true)
	const [collection, setCollection] = useState({
		name: '',
		id: 0,
	});
	const collection_id = props.match.params.id;
	const comment_ref = useRef(null);

	useEffect(() => {
		const url = `/collections/${collection_id}`
		getRequest(url, (data) => { setCollection(Object.assign({}, data)); setLoading(false) }, (error) => { setError(true); setLoading(false) })
	}, [state.currentUser.id]);

	const handleScrollToComments = () => {
		comment_ref.current.scrollIntoView({ behavior: 'smooth' })
	}

	if (error)
		return <CollectionNotFound>
			<Grid container direction='column' justify='center' alignItems='center' spacing={1}>
				<Grid item>
					<p className="not-found">
						A coleção não foi encontrado em nossa base de dados.
					</p>
				</Grid>
				<Grid item>

					<Link className="link" to={`/busca?page=0&results_per_page=12&order=review_average&query=*&search_class=Collection`}>
						<Button
						variant='contained'
						className="back-button"
					>
						Voltar para a busca de coleções.
						</Button>
					</Link>
			</Grid>
			</Grid>
		</CollectionNotFound >
	if (loading)
		return <LoadingSpinner text="Carregando coleção..." />
	else
		return (
			<>
				<BreadCrumbsDiv>
					<StyledBreadCrumbs>
						<Link to="/">Página Inicial</Link>
						<span>Coleções</span>
					</StyledBreadCrumbs>
				</BreadCrumbsDiv>
				<Grid container direction="row" justify="center" alignItems="center">
					<Grid item md={3}>
						<CollectionAuthor
							author_id={collection.owner.id ? collection.owner.id : 0}
							name={collection.owner.name ? collection.owner.name : ""}
							imgsrc={collection.owner.avatar ? apiDomain + collection.owner.avatar : noAvatar} />
					</Grid>


					<Grid item md={5}>
						<CollectionDescription
							stars={collection.review_average}
							likes={collection.likes_count}
							liked={collection.liked}
							scrollToComments={handleScrollToComments}
							title={collection.name ? collection.name : ""}
							collection_id={collection_id} />
					</Grid>

					<Grid item md={3}>
						<DowloadButton
							id={collection_id}
						/>
						<div style={{ height: 12 }}></div>
						<FollowCollectionButton
							followed={collection.followed}
							user_id={state.currentUser.id}
							collection_id={collection_id} />
					</Grid>
				</Grid>

				<VerticalRuler width={1} height={100} color="rgb(238, 238, 238)" />

				<Grid container justify="center" style={{ backgroundColor: '#f4f4f4' }}>
					{/* <Grid item xs={1}/> */}
					<Grid item xs={10}>
						<ResourceList resources={
							collection.collection_items ?
								collection.collection_items.map(i => {
									return {
										type: i.collectionable.object_type,
										author: i.collectionable.author,
										title: i.collectionable.name,
										rating: i.collectionable.review_average,
										likeCount: i.collectionable.likes_count,
										liked: i.collectionable.liked,
										avatar: i.collectionable.publisher.avatar,
										thumbnail: i.collectionable.thumbnail,
										tags: i.collectionable.tags.map(t => t),
										id: i.collectionable.id,
										downloadableLink: i.collectionable.default_attachment_location,
										publisher: i.collectionable.publisher.name,
										published: i.collectionable.state
									}
								})
								: []
						} />

					</Grid>
					<Grid container item xs={12} style={{ marginTop: 40, paddingBottom: 40 }} ref={comment_ref}>
						<CollectionCommentSection
							id={collection_id}
							currentUserId={state.currentUser.id}
							avatar={state.currentUser.avatar ? apiDomain + state.currentUser.avatar : noAvatar}
						/>
					</Grid>
				</Grid>
			</>
		);
}

const StyledBreadCrumbs = styled(Breadcrumbs)`
  display: flex;
  justify-content: flex-start;
  span {
    color: #a5a5a5;
  }
  a {
    color: #00bcd4;
    text-decoration: none;
  }
`;

const CollectionNotFound = styled.div`
  margin: 1em; 
  
  .not-found{
    font-family: 'Roboto', sans-serif;
    font-weight: 500;
    text-align: left; 
    padding: 0; 
    margin: 0; 
  }

  .back-button{
    background-color: #673ab7;
    color:  whitesmoke; 
  }

  .link{
    text-decoration: none;
  }
`

const BreadCrumbsDiv = styled.div`
  padding: 10px;
  display: flex;
`;
